@@ -80,7 +80,7 @@ static ssize_t xattr_setxattr(const char *path, const char *name,
         return -1;
     }
 
-    if (options & XATTR_XATTR_NOFOLLOW) {
+    if (nofollow) {
         rv = extattr_set_link(path, EXTATTR_NAMESPACE_USER,
                                 name, value, size);
     }
@@ -175,7 +175,7 @@ static ssize_t xattr_fsetxattr(int fd, const char *name, void *value,
         return -1;
     }
 
-    if (options & XATTR_XATTR_NOFOLLOW) {
+    if (nofollow) {
         return -1;
     }
     else {
@@ -387,14 +387,19 @@ static ssize_t xattr_xflistxattr(int xfd, char *namebuf, size_t size, int option
     ssize_t nsize = 0;
 
     dirp = fdopendir(xfd);
+    if (dirp == NULL) {
+        return (-1);
+    }
     while (entry = readdir(dirp)) {
-	esize = strlen(entry->d_name);
-	if (nsize + esize + 1 < size) {
-	    strcat(namebuf + nsize, entry->d_name);
+        if (strcmp(entry->d_name, ".") == 0 ||
+                strcmp(entry->d_name, "..") == 0)
+            continue;
+	    esize = strlen(entry->d_name);
+	    if (nsize + esize + 1 <= size) {
+            snprintf((char *)(namebuf + nsize), esize + 1,
+                    entry->d_name);
+    	}
 	    nsize += esize + 1; /* +1 for \0 */
-	} else {
-	    break;
-	}
     }
     closedir(dirp);
     return nsize;
@@ -447,7 +452,7 @@ static ssize_t xattr_setxattr(const char *path, const char *name, void *value, s
     } else if (options != 0) {
         return -1;
     }
-    if (options & XATTR_XATTR_NOFOLLOW) {
+    if (nofollow) {
         return lsetxattr(path, name, value, size, options);
     } else {
         return setxattr(path, name, value, size, options);
